package statocaml_plots

  1. Overview
  2. Docs

Source file colors.ml

1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
(*********************************************************************************)
(*                Statocaml                                                      *)
(*                                                                               *)
(*    Copyright (C) 2025 INRIA All rights reserved.                              *)
(*    Author: Maxence Guesdon (INRIA Saclay)                                     *)
(*      with Gabriel Scherer (INRIA Paris) and Florian Angeletti (INRIA Paris)   *)
(*                                                                               *)
(*    This program is free software; you can redistribute it and/or modify       *)
(*    it under the terms of the GNU General Public License as                    *)
(*    published by the Free Software Foundation, version 3 of the License.       *)
(*                                                                               *)
(*    This program is distributed in the hope that it will be useful,            *)
(*    but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               *)
(*    GNU General Public License for more details.                               *)
(*                                                                               *)
(*    You should have received a copy of the GNU General Public                  *)
(*    License along with this program; if not, write to the Free Software        *)
(*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA                   *)
(*    02111-1307  USA                                                            *)
(*                                                                               *)
(*    As a special exception, you have permission to link this program           *)
(*    with the OCaml compiler and distribute executables, as long as you         *)
(*    follow the requirements of the GNU GPL in regard to all of the             *)
(*    software in the executable aside from the OCaml compiler.                  *)
(*                                                                               *)
(*    Contact: Maxence.Guesdon@inria.fr                                          *)
(*                                                                               *)
(*********************************************************************************)

let colors = [|
   "dark-grey" ;
   "grey30";
   "light-cyan";
   "web-blue";
   "plum";
   "mediumpurple3";
   "seagreen";
   "sea-green";
   "light-pink";
   "red";
   "goldenrod";
   "coral";
   "sienna4";
   "dark-goldenrod";
   "skyblue";
   "turquoise";
   "#5000a0";
   "orchid4";
   "dark-green";
   "forest-green";
   "brown";
   "light-red";
   "midnight-blue";
   "royalblue";
   "dark-violet";
   "purple";
   "web-green";
   "dark-magenta";
   "dark-cyan";
   "dark-orange";
   "dark-blue";
   "dark-yellow";
   "dark-spring-green";
   "steelblue";
   "spring-green";
   "dark-red";
   "dark-chartreuse";
   "orchid";
   "aquamarine";
   "yellow";
   "gold";
   "green";
   "light-coral";
   "navy";
   "beige";
   "olive";
   "blue";
   "cyan";
   "magenta";
   "medium-blue";
   "dark-turquoise";
   "dark-pink";
   "orange-red";
   "salmon";
   "dark-salmon";
   "khaki";
   "orange";
   "dark-khaki";
   "violet";
   "dark-plum";
   "dark-olivegreen";
   "orangered4";
   "slateblue1";
   "yellow4";
   "sienna1";
   "tan1";
   "sandybrown";
   "light-salmon";
   "brown4";
  |]

let get =
  let len = Array.length colors in
  fun i -> colors.(i mod len)