package statocaml_plots

  1. Overview
  2. Docs

Source file json.ml

1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
(*********************************************************************************)
(*                Statocaml                                                      *)
(*                                                                               *)
(*    Copyright (C) 2025 INRIA All rights reserved.                              *)
(*    Author: Maxence Guesdon (INRIA Saclay)                                     *)
(*      with Gabriel Scherer (INRIA Paris) and Florian Angeletti (INRIA Paris)   *)
(*                                                                               *)
(*    This program is free software; you can redistribute it and/or modify       *)
(*    it under the terms of the GNU General Public License as                    *)
(*    published by the Free Software Foundation, version 3 of the License.       *)
(*                                                                               *)
(*    This program is distributed in the hope that it will be useful,            *)
(*    but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               *)
(*    GNU General Public License for more details.                               *)
(*                                                                               *)
(*    You should have received a copy of the GNU General Public                  *)
(*    License along with this program; if not, write to the Free Software        *)
(*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA                   *)
(*    02111-1307  USA                                                            *)
(*                                                                               *)
(*    As a special exception, you have permission to link this program           *)
(*    with the OCaml compiler and distribute executables, as long as you         *)
(*    follow the requirements of the GNU GPL in regard to all of the             *)
(*    software in the executable aside from the OCaml compiler.                  *)
(*                                                                               *)
(*    Contact: Maxence.Guesdon@inria.fr                                          *)
(*                                                                               *)
(*********************************************************************************)

(** *)

type 'data plotter = Plot.gp -> Yojson.Safe.t -> 'data -> unit Lwt.t

let to_plotter : 'a Ocf.wrapper -> 
  (Plot.gp -> 'a -> 'data -> unit Lwt.t) -> 'data plotter =
  fun w plot_fun ->
    let f gp json =
      let param = w.Ocf.Wrapper.from_json json in
      plot_fun gp param
    in
    f

module Smap = Statocaml.Smap

type plot_param =
  { w: int option [@ocf Ocf.Wrapper.(option int), None] ;
    h: int option [@ocf Ocf.Wrapper.(option int), None] ;
    title: string option [@ocf Ocf.Wrapper.(option string), None] ;
    title_font_size: int option [@ocf Ocf.Wrapper.(option int), None] ;
    outfile : string [@ocf Ocf.Wrapper.string, "/tmp/foo.png"] ;
    terminal : string [@ocf Ocf.Wrapper.string, "pngcairo"] ;
    plotter : string [@ocf Ocf.Wrapper.string, ""] ;
  } [@@ocf]

module Register (P:Statocaml_profile.T.S) =
  struct
    let plotters : P.t plotter Smap.t ref = ref Smap.empty
    let add_plotter name f = plotters := Smap.add name f !plotters
    let plotter_by_name name = Smap.find_opt name !plotters

    let plot ?dir p json data =
      match plotter_by_name p.plotter with
      | None -> failwith (Printf.sprintf "No plotter %S" p.plotter)
      | Some plotter ->
          let outfile =
            match dir with
            | None -> p.outfile
            | Some dir -> Filename.concat dir p.outfile
          in
          let gp = Plot.create ~terminal:p.terminal
            ?w:p.w ?h:p.h ?title:p.title ?title_font_size:p.title_font_size
              outfile
          in
          let%lwt () = plotter gp json data in
          Lwt.return (p,gp)

    let plot_from_json ?dir json data =
      let param = plot_param_wrapper.Ocf.Wrapper.from_json json in
      plot ?dir param json data
  end